<?php

namespace backend\modules\reports\controllers;

use backend\modules\reports\models\SalesCustomerLedger;
use backend\modules\reports\models\SalesCustomerLedgerSearch;
use backend\modules\master\models\Customer;
use backend\modules\users\models\AccountingDetails;
use backend\modules\users\models\AccountingJournal;
use backend\modules\users\models\CreditNote;
use backend\modules\users\models\ReceiptDetails;
use backend\modules\users\models\DebitNote;
use yii\data\ActiveDataProvider;
use Yii;
use yii\helpers\ArrayHelper;
use Mpdf\Mpdf;
use backend\modules\users\models\Invoice;


class SalesCustomerLedgerController extends \yii\web\Controller
{
  public function actionIndex($excel = null, $print = null, $detailprint = null, $detailexcel = null)
  {
    $model = new SalesCustomerLedger();
    $searchModel = new SalesCustomerLedgerSearch();
    $query = SalesCustomerLedger::find();
    if (Yii::$app->request->get()) {
      $searchModel->load(Yii::$app->request->queryParams);
      $query->andFilterWhere(['like', 'ledgerhead_id', $searchModel->ledgerhead_id])
        // ->andFilterWhere(['between', 'Date', $searchModel->from_date,$searchModel->to_date])
        ->andFilterWhere(['like', 'executive', $searchModel->executive])
        ->andFilterWhere(['like', 'customer_type', $searchModel->customer_type])
        ->andFilterWhere(['like', 'account_type_id', $searchModel->account_type_id])
        ->andFilterWhere(['like', 'group_name_id', $searchModel->group_name_id]);

      if ($searchModel->order_by) {
        if ($searchModel->order_by == 'acidasc') {
          $query->orderBy(['ledgerhead_id' => SORT_ASC]);
        } else {
          $query->orderBy(['ledgerhead_id' => SORT_DESC]);
        }
      }
      if ($searchModel->ledger_balance) {
        if ($searchModel->ledger_balance == 'zero') {
          $query->andWhere(['and', ['total_debit_amount' => 0], ['total_credit_amount' => 0]]);
        } else {
          $query->orFilterWhere(['>', 'total_debit_amount', 0])
            ->orFilterWhere(['>', 'total_credit_amount', 0]);
        }
      }
    }
    $dataProvider = new ActiveDataProvider([
      'query' => $query,
      'pagination' => false,

    ]);
    if ($detailprint != null) {
      $for = "detail";
      $this->actionExportpdf($dataProvider, $searchModel, $for);
    }
    if ($detailexcel != null) {
      $this->actionExceldetailreport($dataProvider, $searchModel);
    }
    if ($excel != null) {
      $this->actionExcelreport($dataProvider, $searchModel);
    }
    if ($print != null) {
      $for = "summary";
      $this->actionExportpdf($dataProvider, $searchModel, $for);
    }
    return $this->render('index', [
      'searchModel' => $searchModel,
      'model' => $model,
      'dataProvider' => $dataProvider,

    ]);
  }

  public function actionExportpdf($data = null, $model = null, $for = null)
  {
    if ($for == 'detail') {
      $data->pagination = false;
      $mpdf = new mPDF;
      $mpdf->showImageErrors = true;
      $mpdf->WriteHTML($this->renderPartial('/sales-customer-ledger/exportpdf', [
        'dataProvider' => $data,
        'searchModel' => $model,

      ]));
      $mpdf->Output('Sale-Customer-Ledger.pdf', 'D');
    } else {
      $data->pagination = false;
      $mpdf = new mPDF;
      $mpdf->showImageErrors = true;
      $mpdf->WriteHTML($this->renderPartial('/sales-customer-ledger/summaryexportpdf', [
        'dataProvider' => $data,
        'searchModel' => $model,

      ]));
      $mpdf->Output('Sale-Customer-Ledger.pdf', 'D');
    }
  }
  public function actionExceldetailreport($data = null, $mod = null)
  {
    // $dataProvider = new ActiveDataProvider([
    //     'query' => PurchaseVendorLedger::find()->orderBy(['Date' => SORT_DESC]),
    // ]);
    if ($data != null) {
      $dataProvider = $data;
    } else {
      $dataProvider = new ActiveDataProvider([
        'query' => SalesCustomerLedger::find()->orderBy(['Date' => SORT_DESC]),
      ]);
    }

    $phpExcel = new \PHPExcel;
    $phpExcel->getDefaultStyle()->getFont()->setName('Calibri');
    $phpExcel->getProperties()->setDescription("Excel SpreadSheet in PHP");
    $writer = \PHPExcel_IOFactory::createWriter($phpExcel, "Excel2007");
    $sheet = $phpExcel->getActiveSheet();
    $row = 1;
    foreach ($dataProvider->models as $model) {
      $sheet->getCell('A' . $row)->setValue('MOHAPATRA SCIENTIFIC PRIVATE LIMITED');
      $sheet->mergeCells("A" . ($row) . ":I" . ($row));
      $sheet->getCell('A' . ($row + 1))->setValue('F2G/30, INDRADHANU MARKET, IRC VILLAGE,');
      $sheet->mergeCells("A" . ($row + 1) . ":I" . ($row + 1));
      $sheet->getCell('A' . ($row + 2))->setValue('BHUBANESWAR, Odisha');
      $sheet->mergeCells("A" . ($row + 2) . ":I" . ($row + 2));

      $sheet->getStyle('A' . $row)->getFont()->setBold(true)->setSize(12);
      $sheet->getStyle('A' . $row)->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
      $sheet->getStyle('A' . ($row + 1))->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
      $sheet->getStyle('A' . ($row + 2))->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);

      $sheet->getCell('A' . ($row + 3))->setValue('');
      $sheet->mergeCells("A" . ($row + 3) . ":I" . ($row + 3));

      $model_address = Customer::find()->where(['business_name' => $model->ledgerhead_id])->one();

      $sheet->getCell('A' . ($row + 4))->setValue($model->ledgerhead_id);
      $sheet->getStyle('A' . ($row + 4))->getFont()->setBold(true)->setSize(10);
      $sheet->mergeCells("A" . ($row + 4) . ":F" . ($row + 4));
      $sheet->getCell('A' . ($row + 5))->setValue($model_address->adderss_line_1 . ',' . $model_address->adderss_line_2 . ',' . $model_address->adderss_line_3);
      $sheet->mergeCells("A" . ($row + 5) . ":F" . ($row + 5));
      $sheet->getCell('A' . ($row + 6))->setValue($model_address->city);
      $sheet->mergeCells("A" . ($row + 6) . ":F" . ($row + 6));
      $sheet->getCell('A' . ($row + 7))->setValue($model_address->state);
      $sheet->mergeCells("A" . ($row + 7) . ":F" . ($row + 7));
      $sheet->getCell('A' . ($row + 8))->setValue($model_address->country . '-' . $model_address->pin_code);
      $sheet->mergeCells("A" . ($row + 8) . ":F" . ($row + 8));
      $sheet->getCell('A' . ($row + 9))->setValue('Phone No-' . (string)$model_address->phone);
      $sheet->mergeCells("A" . ($row + 9) . ":F" . ($row + 9));
      $sheet->mergeCells("G" . ($row + 4) . ":G" . ($row + 9));
      $sheet->getCell('H' . ($row + 4))->setValue('STATEMENT OF ACCOUNT');
      $sheet->getStyle('H' . ($row + 4))->getAlignment()->setVertical(\PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_TOP);
      $sheet->getStyle('H' . ($row + 4))->getFont()->setBold(true)->setSize(10);
      $sheet->mergeCells("H" . ($row + 4) . ":H" . ($row + 9));

      if ($mod->from_date != null && $mod->to_date != '') {
        $sheet->getCell('A' . ($row + 10))->setValue(date('d/m/Y', strtotime($mod->from_date)) . ' to ' . date('d/m/Y', strtotime($mod->to_date)));
      } else {
        $sheet->getCell('A' . ($row + 10))->setValue('');
      }
      $sheet->mergeCells("A" . ($row + 10) . ":F" . ($row + 10));


      $sheet->getCell('A' . ($row + 11))->setValue('Date');
      $sheet->getColumnDimension('A')->setWidth(20);
      $sheet->getCell('B' . ($row + 11))->setValue('REF.NO.');
      $sheet->getColumnDimension('B')->setWidth(20);
      $sheet->getCell('C' . ($row + 11))->setValue('DESCRIPTION');
      $sheet->getColumnDimension('C')->setWidth(40);
      $sheet->getCell('D' . ($row + 11))->setValue('NOTES');
      $sheet->getColumnDimension('D')->setWidth(40);
      $sheet->getCell('E' . ($row + 11))->setValue('VCH TYPE');
      $sheet->getColumnDimension('E')->setWidth(10);
      $sheet->getCell('F' . ($row + 11))->setValue('DEBIT');
      $sheet->getColumnDimension('F')->setWidth(15);
      $sheet->getCell('G' . ($row + 11))->setValue('CREDIT');
      $sheet->getColumnDimension('G')->setWidth(15);
      $sheet->getCell('H' . ($row + 11))->setValue('BALANCE');
      $sheet->getColumnDimension('H')->setWidth(22);

      // $balance = Customer::find()->where(['business_name' => $model->ledgerhead_id])->one()->opening_as_on;
      $balance = 0;


      $sheet->getCell('E' . ($row + 12))->setValue('Opening');


      // $model_details = AccountingDetails::find()->where(['ledgerhead_id'=>$model->ledgerhead_id])->all();
      if ($mod->from_date != '' && $mod->to_date != '') {

        $model_details = AccountingDetails::find()->where(['ledgerhead_id' => $model->ledgerhead_id])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Invoice'], ['transaction_type' => 'Receipt'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note'],['transaction_type' => 'Journal']])->andFilterWhere(['>=', 'date', $mod->from_date])->andFilterWhere(['<=', 'date', $mod->to_date])->orderBy(['date' => SORT_ASC])->all();
      } else {
        $model_details = AccountingDetails::find()->where(['ledgerhead_id' => $model->ledgerhead_id])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Invoice'], ['transaction_type' => 'Receipt'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note'],['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC])->all();
      }




      $i = $row + 13;
      $row_amount = 0;

      $Debit_value = 0;
      $Credit_value = 0;

      if ($mod->from_date != '' && $mod->to_date != '') {
        $prev_debit_amount = 0;
        $previous_debit = AccountingDetails::find()->andFilterWhere(['<=', 'date', $mod->from_date])
          ->andFilterWhere(['edit_status' => 0])
          ->andFilterWhere(['in', 'ledgerhead_id', $model->ledgerhead_id])
          ->andFilterWhere(['or', ['transaction_type' => 'Receipt'], ['transaction_type' => 'Debit Note'],['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC])->all();
        foreach ($previous_debit as $key => $prev_debit) {
          $prev_debit_amount = $prev_debit_amount + $prev_debit->amount;
        }

        $prev_credit_amount = 0;
        $previous_credit = AccountingDetails::find()->andFilterWhere(['<=', 'date', $mod->from_date])
          ->andFilterWhere(['edit_status' => 0])
          ->andFilterWhere(['in', 'ledgerhead_id', $model->ledgerhead_id])
          ->andFilterWhere(['or', ['transaction_type' => 'Invoice'], ['transaction_type' => 'Credit Note']])->orderBy(['date' => SORT_ASC])->all();
        foreach ($previous_credit as $key => $prev_credit) {
          $prev_credit_amount = $prev_credit_amount + $prev_credit->amount;
        }
        $before_balance = (($balance + $prev_credit_amount) - $prev_debit_amount);

        $sheet->getCell('H' . ($row + 12))->setValue((string)$before_balance);
      } else {
        $sheet->getCell('H' . ($row + 12))->setValue($balance);
      }


      foreach ($model_details as $key => $val) {

        if ($mod->from_date != '' && $mod->to_date != '') {
          $prev_debit_amount = 0;
          $previous_debit = AccountingDetails::find()->andFilterWhere(['<=', 'date', $mod->from_date])
            ->andFilterWhere(['edit_status' => 0])
            ->andFilterWhere(['in', 'ledgerhead_id', $val->ledgerhead_id])
            ->andFilterWhere(['or', ['transaction_type' => 'Receipt'], ['transaction_type' => 'Debit Note'],['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC])->all();
          foreach ($previous_debit as $key => $prev_debit) {
            $prev_debit_amount = $prev_debit_amount + $prev_debit->amount;
          }
          $sheet->getCell('F' . ($row + 12))->setValue((string)$prev_debit_amount);
        } else {
          $sheet->getCell('F' . ($row + 12))->setValue('0');
        }

        if ($mod->from_date != '' && $mod->to_date != '') {
          $prev_debit_amount = 0;
          $previous_debit = AccountingDetails::find()->andFilterWhere(['<=', 'date', $mod->from_date])
            ->andFilterWhere(['edit_status' => 0])
            ->andFilterWhere(['in', 'ledgerhead_id', $val->ledgerhead_id])
            ->andFilterWhere(['or', ['transaction_type' => 'Invoice'], ['transaction_type' => 'Credit Note']])->orderBy(['date' => SORT_ASC])->all();
          foreach ($previous_debit as $key => $prev_debit) {
            $prev_debit_amount = $prev_debit_amount + $prev_debit->amount;
          }
          $sheet->getCell('G' . ($row + 12))->setValue((string)$prev_debit_amount);
        } else {
          $sheet->getCell('G' . ($row + 12))->setValue('0');
        }
        $count = 0;
        if ($val != '')
          $total_trancection = AccountingDetails::find()->where(['transaction_id' => $val->transaction_id])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Invoice'], ['transaction_type' => 'Receipt'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note'],['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC])->all();
        $row_amount = $row_amount + count($total_trancection);
        $count = count($total_trancection);

        if ($row_amount > 1)
          $sheet->mergeCells("A" . ($i) . ":A" . ($i + ($count - 2)));
        $sheet->mergeCells("B" . ($i) . ":B" . ($i + ($count - 2)));
        $sheet->mergeCells("D" . ($i) . ":D" . ($i + ($count - 2)));
        $sheet->mergeCells("E" . ($i) . ":E" . ($i + ($count - 2)));
        $sheet->mergeCells("F" . ($i) . ":F" . ($i + ($count - 2)));
        $sheet->mergeCells("G" . ($i) . ":G" . ($i + ($count - 2)));
        $sheet->mergeCells("H" . ($i) . ":H" . ($i + ($count - 2)));

        foreach ($total_trancection as $key => $value) {

          if ($value->accounting_type == 'Credit' && $value->ledgerhead_id == $val->ledgerhead_id) {
            //&& $value->ledgerhead_id==$val->ledgerhead_id
            $Credit_value = $Credit_value + $val->amount;
            if ($mod->from_date != '' && $mod->to_date != '') {
              if ($before_balance != '') {
                $before_balance = (float)$before_balance +  $val->amount;
              } else {
                $before_balance = 0 + $val->amount;
              }
            } else {

              if ($balance != '') {
                $balance = (float)$balance + $val->amount;
              } else {
                $balance = 0 + $val->amount;
              }
            }
          } else if ($value->accounting_type == 'Debit' && $value->ledgerhead_id == $val->ledgerhead_id && $value->transaction_type != 'Credit Note') {
            if ($mod->from_date != '' && $mod->to_date != '') {
              if ($before_balance != '') {
                $before_balance = (float)$before_balance - $val->amount;
              } else {
                $before_balance = 0 - $val->amount;
              }
            } else {
              if ($balance != '') {
                // if($balance<0)
                //  {
                //    $balance = $balance * -1;
                //    $balance = (float)$balance-$val->amount;

                //   }
                // else{
                //  $balance = (float)$balance-$val->amount;
                //     }
                //	if($balance>$val->amount)
                $balance = (float)$balance - $val->amount;
                //      else
                //	   $balance = $val->amount-(float)$balance;	
              } else {
                $balance = 0 - $val->amount;
              }
            }

            $Debit_value = $Debit_value + $val->amount;
          } else if ($value->accounting_type == 'Debit' && $value->ledgerhead_id == $val->ledgerhead_id && $value->transaction_type == 'Credit Note') {
            $Credit_value = $Credit_value + $val->amount;
            if ($mod->from_date != '' && $mod->to_date != '') {
              if ($before_balance != '') {
                $before_balance = (float)$before_balance + $val->amount;
              } else {
                $before_balance = 0 + $val->amount;
              }
            } else {

              if ($balance != '') {
                //	  if($balance>$val->amount)
                $balance = (float)$balance + $val->amount;
                //	else
                //	    $balance = $val->amount+(float)$balance;
              } else {
                $balance = 0 + $val->amount;
              }
            }
          }


          if ($value->ledgerhead_id != $val->ledgerhead_id) {
            $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->date)));

            if ($value->transaction_type == 'Receipt') {
              $findbill = ReceiptDetails::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $sheet->getCell('B' . ($i))->setValue($findbill->receipt_id);
                $sheet->getStyle('D' . ($i))->getAlignment()->setWrapText(true);
                //		 $sheet->getColumnDimension('A')->setRowHeight(40);
                $sheet->getCell('D' . ($i))->setValue($findbill->notes);
              } else
                $sheet->getCell('B' . ($i))->setValue('');
            } elseif ($value->transaction_type == 'Credit Note') {
              $findbill = CreditNote::find()->where(['credit_note_id' => $value->transaction_id])->one();
              // if($findbill)
              //  $sheet ->getCell('B' . ($i))->setValue($findbill->credit_note_id);
              // else
              //   $sheet ->getCell('D' . ($i))->setValue($findbill->notes);

              $sheet->getCell('B' . ($i))->setValue($value->transaction_id);
            } elseif ($value->transaction_type == 'Debit Note') {
              $findbill = DebitNote::find()->where(['id' => $value->transaction_id])->one();
              $sheet->getCell('B' . ($i))->setValue($findbill->prefix . '/' . $findbill->po . '/' . $findbill->suffix);
              $sheet->getCell('D' . ($i))->setValue($findbill->notes);
            } 
            elseif ($value->transaction_type == 'Journal') {
              $str = ltrim($value->transaction_id, 'J');
              $findbill = AccountingJournal::find()->where(['jv_no' =>$str])->one();
              $sheet->getCell('B' . ($i))->setValue($value->transaction_id);
              $sheet->getCell('D' . ($i))->setValue($findbill->notes);
            }  else {
              $sheet->getCell('B' . ($i))->setValue($value->transaction_id);
              $findbill = Invoice::find()->where(['invoice_id' => $value->transaction_id])->one();
              //$sheet ->getCell('D' . ($i))->setValue($findbill->notes);
            }

            $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
            // $sheet ->getCell('D' . ($i))->setValue($val->ledgerhead_id);
            $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
            if ($value->transaction_type == 'Receipt' || $value->transaction_type == 'Credit Note') {
              $sheet->getCell('G' . ($i))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $val->amount));
              $sheet->getStyle('F' . ($i))->getNumberFormat()->setFormatCode('0.00');
              $sheet->getCell('F' . ($i))->setValue("0.00");
            } 
            elseif( $value->transaction_type == 'Journal')
            {
              if($value->accounting_type=='Debit')
              {
                $sheet->getCell('G' . ($i))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $val->amount));
                $sheet->getStyle('F' . ($i))->getNumberFormat()->setFormatCode('0.00');
                $sheet->getCell('F' . ($i))->setValue("0.00");
              }
              else{
                $sheet->getCell('F' . ($i))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $val->amount));
                $sheet->getStyle('G' . ($i))->getNumberFormat()->setFormatCode('0.00');
                $sheet->getCell('G' . ($i))->setValue("0.00");
              }
              
            }else {
              $sheet->getStyle('G' . ($i))->getNumberFormat()->setFormatCode('0.00');
              $sheet->getCell('G' . ($i))->setValue("0.00");
              $sheet->getCell('F' . ($i))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $val->amount));
            }
          } else if ($value->ledgerhead_id == $val->ledgerhead_id) {
            if ($mod->from_date != '' && $mod->to_date != '') {
              $sheet->getCell('H' . (($i)))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $before_balance));
            } else {
              $sheet->getCell('H' . (($i)))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $balance));
            }

            $sheet->getStyle('H' . ($i))->getFont()->setBold(true)->setSize(10);
            // $total_balance = $total_balance + $balance;
          }
          $i++;
        }
      }
      $sheet->getCell('E' . ($row + 13 + $row_amount))->setValue('Total (Rs)');
      $sheet->getStyle('E' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);
      $sheet->getStyle('F' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
      $sheet->getCell('F' . ($row + 13 + $row_amount))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $Debit_value));
      $sheet->getStyle('F' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);
      $sheet->getStyle('G' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
      $sheet->getCell('G' . ($row + 13 + $row_amount))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $Credit_value));
      $sheet->getStyle('G' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);
      if ($mod->from_date != '' && $mod->to_date != '') {
        $sheet->getStyle('H' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
        $sheet->getCell('H' . ($row + 13 + $row_amount))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $before_balance));
      } else {
        $sheet->getStyle('H' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
        $sheet->getCell('H' . ($row + 13 + $row_amount))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $balance));
      }

      $sheet->getStyle('H' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);


      $row = $row + 16 + $row_amount;
    }
    // die();

    ob_clean();
    ob_start();

    header('Content-type: application/vnd.ms-excel'); // It will be called file.xls  
    header('Content-Disposition: attachment; filename="sale-customer-ledger-detailed.xls"'); // Write file to the browser 

    $writer->save('php://output');
    ob_flush();
    exit();
  }
  public function actionExcelreport($data = null, $mod = null)
  {
    // $dataProvider = new ActiveDataProvider([
    //     'query' => PurchaseVendorLedger::find()->orderBy(['Date' => SORT_DESC]),
    // ]);
    if ($data != null) {
      $dataProvider = $data;
    } else {
      $dataProvider = new ActiveDataProvider([
        'query' => SalesCustomerLedger::find()->orderBy(['Date' => SORT_DESC]),
      ]);
    }

    $phpExcel = new \PHPExcel;
    $phpExcel->getDefaultStyle()->getFont()->setName('Calibri');
    // $phpExcel->getDefaultStyle()->getFont()->setSize(14);
    // $phpExcel ->getProperties()->setTitle("Vendor list");
    $phpExcel->getProperties()->setDescription("Excel SpreadSheet in PHP");
    $writer = \PHPExcel_IOFactory::createWriter($phpExcel, "Excel2007");
    $sheet = $phpExcel->getActiveSheet();

    // $sheet->setTitle('My product list');
    $sheet->getCell('A1')->setValue('MOHAPATRA SCIENTIFIC PRIVATE LIMITED');
    $sheet->getCell('A2')->setValue('F2G/30, INDRADHANU MARKET, IRC VILLAGE,');
    $sheet->getCell('A3')->setValue('BHUBANESWAR, Odisha');
    $sheet->getCell('A4')->setValue('');
    $sheet->getCell('A5')->setValue('Ledgers');
    $sheet->getStyle('A5')->getFont()->setBold(true)->setSize(11);


    $sheet->getStyle('A1')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    $sheet->getStyle('A2')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    $sheet->getStyle('A3')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    $sheet->getStyle('A4')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    $sheet->getStyle('A5')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    $sheet->getStyle('A6')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);

    // Making headers text bold and larger
    $sheet->getStyle('A1:S1')->getFont()->setBold(true)->setSize(12);
    $sheet->getStyle('A2:S2')->getFont()->setSize(11);
    $sheet->getStyle('A3:S3')->getFont()->setSize(11);
    $sheet->getStyle('A4:S4')->getFont()->setSize(11);
    $sheet->getStyle('A5:S5')->getFont()->setSize(11);
    $sheet->getStyle('A6:S6')->getFont()->setSize(11);

    $sheet->getColumnDimension('A')->setWidth(60);
    $sheet->getColumnDimension('B')->setWidth(10);
    $sheet->getColumnDimension('C')->setWidth(10);

    $sheet->mergeCells('A1:C1');
    $sheet->mergeCells('A2:C2');
    $sheet->mergeCells('A3:C3');
    $sheet->mergeCells('A4:C4');
    $sheet->mergeCells('A5:C5');
    $sheet->mergeCells('A6:C6');

    if ($mod->from_date != null) {
      $sheet->setCellValue('A6', date('d/m/Y', strtotime($mod->from_date)) . ' to ' . date('d/m/Y', strtotime($mod->to_date)));
    } else {
      $sheet->setCellValue('A6', '');
    }


    $sheet->setCellValue('A7',  'Ledger Name');
    $sheet->setCellValue('B7',  'Debit');
    $sheet->setCellValue('C7',  'Credit');
    $sheet->getStyle('A7')->getFont()->setBold(true)->setSize(11);
    $sheet->getStyle('B7')->getFont()->setBold(true)->setSize(11);
    $sheet->getStyle('C7')->getFont()->setBold(true)->setSize(11);

    $row = 8;
    $total_debit_amount = 0;
    $total_credit_amount = 0;
    foreach ($dataProvider->models as $model) {





      if ($mod->from_date != '' && $mod->from_date != '') {
        $amount = 0;
        $debit = AccountingDetails::find()->andFilterWhere(['in', 'ledgerhead_id', $model->ledgerhead_id])->andFilterWhere(['edit_status' => 0])->andFilterWhere(['or', ['transaction_type' => 'Invoice'], ['transaction_type' => 'Credit Note']])->andFilterWhere(['>=', 'date', $mod['from_date']])->andFilterWhere(['<=', 'date', $mod['to_date']])->all();
        foreach ($debit as $key => $value) {

          if ($value->amount != '') {
            $amount = (float)$amount + $value->amount;
          } else {
            $amount = 0 + $value->amount;
          }
        }

        $credit_amt = 0;
        $credit = AccountingDetails::find()->andFilterWhere(['in', 'ledgerhead_id', $model->ledgerhead_id])->andFilterWhere(['edit_status' => 0])->andFilterWhere(['or', ['transaction_type' => 'Receipt'], ['transaction_type' => 'Debit Note']])->andFilterWhere(['>=', 'date', $mod['from_date']])->andFilterWhere(['<=', 'date', $mod['to_date']])->all();
        foreach ($credit as $key => $value) {

          if ($value->amount != '') {
            $credit_amt = (float)$credit_amt + $value->amount;
          } else {
            $credit_amt = 0 + $value->amount;
          }
        }
        $tot_deb = $amount - $credit_amt;
        $tot_cre = $credit_amt - $amount;
        $sheet->setCellValue('B' . $row,  (string)$tot_deb);
        $sheet->setCellValue('C' . $row,  (string)$tot_cre);

        // $sheet->setCellValue('C' . $row,  (string)$credit_amount-$amount);  

        $total_credit_amount = $total_credit_amount + $credit_amt;
        $total_debit_amount = $total_debit_amount + $amount;
      } else {
        $total_debit = $model->total_debit_amount - $model->total_credit_amount;
        $total_credit = $model->total_credit_amount - $model->total_debit_amount;
        $sheet->setCellValue('B' . $row,  (string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $total_debit));
        $sheet->setCellValue('C' . $row,  (string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $total_credit));

        $total_debit_amount += $model->total_debit_amount - $model->total_credit_amount;
        $total_credit_amount += $model->total_credit_amount - $model->total_debit_amount;
      }


      $sheet->setCellValue('A' . $row,  $model->ledgerhead_id);

      $row++;
    }

    $sheet->setCellValue('A' . $row, 'Total');
    $sheet->setCellValue('B' . $row,  (string)$total_debit_amount);
    $sheet->setCellValue('C' . $row,  (string)$total_credit_amount);

    $sheet->getStyle('A' . $row)->getFont()->setBold(true)->setSize(11)->getColor()->setRGB('FF0000');
    $sheet->getStyle('B' . $row)->getFont()->setBold(true)->setSize(11)->getColor()->setRGB('FF0000');
    $sheet->getStyle('C' . $row)->getFont()->setBold(true)->setSize(11)->getColor()->setRGB('FF0000');

    ob_clean();
    ob_start();

    header('Content-type: application/vnd.ms-excel'); // It will be called file.xls  
    header('Content-Disposition: attachment; filename="sales-customer-ledger.xls"'); // Write file to the browser 

    $writer->save('php://output');
    ob_flush();
    exit();
  }


  public function actionCustomerdetails($id = null, $totalexcel = null)
  {
    $model = new SalesCustomerLedger();
    $customer = Customer::find()->where(['id' => $id])->one();
    $query = AccountingDetails::find()->where(['ledgerhead_id' => $customer->business_name])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Invoice'], ['transaction_type' => 'Receipt'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note'], ['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC]);
    // $query = AccountingDetails::find()->where(['ledgerhead_id' => $vendor->business_name])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note']])->orderBy(['date' => SORT_ASC]);
    if ($model->load(Yii::$app->request->get())) {
      $query->andFilterWhere(['between', 'date', $model->details_from_date, $model->details_to_date])
        ->andFilterWhere(['like', 'transaction_type', $model->details_type])
        ->andFilterWhere(['between', 'amount', $model->details_total_from, $model->details_total_to]);
      // ->andFilterWhere(['between', 'Date', $model->from_date,$model->to_date]);    
    }
    $dataProvider = new ActiveDataProvider([
      'query' => $query,
      'pagination' => false
    ]);
    // if ($totalexcel != null) {
    //   // die();
    //   // AccountingDetails::find()->where(['ledgerhead_id'=>$model->ledgerhead_id])->andWhere(['edit_status'=>0])->andWhere(['or',['transaction_type'=>'Bill'],['transaction_type'=>'Payment'],['transaction_type'=>'Credit Note'],['transaction_type'=>'Debit Note']])->orderBy(['date' => SORT_ASC ])->all();

    //   $query2 = AccountingDetails::find()->where(['ledgerhead_id' => $vendor->business_name])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note']])->orderBy(['date' => SORT_ASC]);
    //   $dataProvider2 = new ActiveDataProvider([
    //     'query' => $query2,
    //     'pagination' => false
    //   ]);
    //   $this->actionExcelvendordetailreport($dataProvider2, $model, $dataProvider2);
    // }

    return $this->render('customerdetails', [
      'model' => $model,
      'customer_id' => $id,
      'dataProvider' => $dataProvider,
      'user' => $customer->business_name,
    ]);
  }


  // public function actionVendordetails($id = null, $totalexcel = null)
  // {
  //   $model = new PurchaseVendorLedger();
  //   $vendor = Vendor::find()->where(['id' => $id])->one();
  //   $query = AccountingDetails::find()->where(['ledgerhead_id' => $vendor->business_name])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note']])->orderBy(['date' => SORT_ASC]);
  //   if ($model->load(Yii::$app->request->get())) {
  //     $query->andFilterWhere(['between', 'date', $model->details_from_date, $model->details_to_date])
  //       ->andFilterWhere(['like', 'transaction_type', $model->details_type])
  //       ->andFilterWhere(['between', 'amount', $model->details_total_from, $model->details_total_to]);
  //     // ->andFilterWhere(['between', 'Date', $model->from_date,$model->to_date]);    
  //   }
  //   $dataProvider = new ActiveDataProvider([
  //     'query' => $query,
  //     'pagination' => false
  //   ]);



  //   if ($totalexcel != null) {
  //     // die();
  //     // AccountingDetails::find()->where(['ledgerhead_id'=>$model->ledgerhead_id])->andWhere(['edit_status'=>0])->andWhere(['or',['transaction_type'=>'Bill'],['transaction_type'=>'Payment'],['transaction_type'=>'Credit Note'],['transaction_type'=>'Debit Note']])->orderBy(['date' => SORT_ASC ])->all();

  //     $query2 = AccountingDetails::find()->where(['ledgerhead_id' => $vendor->business_name])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note']])->orderBy(['date' => SORT_ASC]);
  //     $dataProvider2 = new ActiveDataProvider([
  //       'query' => $query2,
  //       'pagination' => false
  //     ]);
  //     $this->actionExcelvendordetailreport($dataProvider2, $model, $dataProvider2);
  //   }

  //   return $this->render('vendordetails', [
  //     'model' => $model,
  //     'vendor_id' => $id,
  //     'dataProvider' => $dataProvider,
  //     'user' => $vendor->business_name,
  //   ]);
  // }


}
