<?php

namespace backend\modules\reports\models;

use Yii;

/**
 * This is the model class for table "gstr1_details".
 *
 * @property string|null $invoice_id
 * @property string|null $gstn_no
 * @property string|null $invoice_date
 * @property float|null $total_value
 * @property float|null $tax_rate
 * @property float|null $taxable_value
 * @property float|null $igst_value
 * @property float|null $cgst_value
 * @property float|null $sgst_amount
 * @property string|null $point_of_supply
 */
class Gstr1Details extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'gstr1_details';
    }

    public $from_date;
    public $to_date;

    public $setting_type;
    public $gst_number;
    public $hsn_no;
    public $bill_type;
    public $supply_type;
    public $reverse_change;
    public $location;
    public $pro_assesment;
    public $find_product;
    public $find_customer;
    public $time_frame;
    public $to;
    public $from;
    public $customer_type;
    public $vendor;

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['invoice_date','from_date','to_date','to','from'], 'safe'],
            [['total_value', 'tax_rate', 'taxable_value', 'igst_value', 'cgst_value', 'sgst_amount'], 'number'],
            [['invoice_id', 'gstn_no', 'point_of_supply'], 'string', 'max' => 254],
            [['setting_type', 'hsn_no','bill_type','supply_type','reverse_change','pro_assesment','find_product','find_customer','time_frame','location','customer_type','vendor'], 'string'],
            [['to_date'], 'compare' ,'compareAttribute' => 'from_date','operator'=>'>=', 'message'=> 'To month must be equal/greater than From month'],

            [['gst_number'], 'string','min'=>15, 'max' => 15],
             [['gst_number'], 'match' ,'pattern'=>'/^([0-9]){2}([A-Z]){4}([0-9A-Z]){1}([0-9]){4}([A-Z]){1}([0-9A-Z]){1}([0-9A-Z]){1}([0-9A-Z]){1}?$/', 'message'=> 'GSTN Format should be like  12ABCDE1234Z1Z1.'],
            [['to'], 'compare' ,'compareAttribute' => 'from','operator'=>'>=', 'message'=> 'To date must be equal/greater than From date'], 
            //  ['to_date', 'required', 'when' => function($model) {
            //     return $model->from_date;
            // },'whenClient' => "function (attribute, value) {
            //     return $('#gstr1details-from_date').val();
            // }"],
            // ['from_date', 'required', 'when' => function($model) {
            //     return $model->to_date;
            // },'whenClient' => "function (attribute, value) {
            //     return $('#gstr1details-to_date').val();
            // }"],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'invoice_id' => 'Invoice ID',
            'gstn_no' => 'Gstn No',
            'invoice_date' => 'Invoice Date',
            'total_value' => 'Total Value',
            'tax_rate' => 'Tax Rate',
            'taxable_value' => 'Taxable Value',
            'igst_value' => 'Igst Value',
            'cgst_value' => 'Cgst Value',
            'sgst_amount' => 'Sgst Amount',
            'point_of_supply' => 'Point Of Supply',
        ];
    }

}
